@@ -16,8 +16,8 @@
 #' * Supply `call` to inform users about which function the error
 #'   occurred in.
 #'
-#' * Supply another condition as `parent` to create a chained
-#'   condition.
+#' * Supply another condition as `parent` to create a [chained
+#'   condition][topic-error-chaining].
 #'
 #' Certain components of condition messages are formatted with unicode
 #' symbols and terminal colours by default. These aspects can be
@@ -42,6 +42,7 @@
 #'   `class` argument. You may consider prefixing condition fields
 #'   with the name of your package or organisation to prevent name
 #'   collisions.
+#' @param body,footer Additional bullets.
 #' @param call The execution environment of a currently running
 #'   function, e.g. `call = caller_env()`. The corresponding function
 #'   call is retrieved and mentioned in error messages as the source
@@ -53,7 +54,26 @@
 #'
 #'   Can also be `NULL` or a [defused function call][topic-defuse] to
 #'   respectively not display any call or hard-code a code to display.
-#' @param body,footer Additional bullets.
+#'
+#'   For more information about error calls, see `r link("topic_error_call")`.
+#' @param parent Supply `parent` when you rethrow an error from a
+#'   condition handler (e.g. with [try_fetch()]).
+#'
+#'   - If `parent` is a condition object, a _chained error_ is
+#'     created, which is useful when you want to enhance an error with
+#'     more details, while still retaining the original information.
+#'
+#'   - If `parent` is `NA`, it indicates an unchained rethrow, which
+#'     is useful when you want to take ownership over an error and
+#'     rethrow it with a custom message that better fits the
+#'     surrounding context.
+#'
+#'     Technically, supplying `NA` lets `abort()` know it is called
+#'     from a condition handler. This helps it create simpler
+#'     backtraces where the condition handling context is hidden by
+#'     default.
+#'
+#'   For more information about error calls, see `r link("topic_error_chaining")`.
 #' @param use_cli_format Whether to format `message` lazily using
 #'   [cli](https://cli.r-lib.org/) if available. This results in
 #'   prettier and more accurate formatting of messages. See
@@ -151,6 +171,10 @@
 #'   especially easy to hit when the message contains a lot of ANSI
 #'   escapes, as created by the crayon or cli packages
 #'
+#' @seealso
+#' - `r link("topic_error_call")`
+#' - `r link("topic_error_chaining")`
+#'
 #' @examples
 #' # These examples are guarded to avoid throwing errors
 #' if (FALSE) {
@@ -228,17 +252,37 @@ abort <- function(message = NULL,
   .__signal_frame__. <- TRUE
   caller <- caller_env()
 
+  rethrowing <- !is_null(parent)
+  if (is_na(parent)) {
+    parent <- NULL
+  }
+
+  if (is_list(maybe_missing(call))) {
+    if (!identical(names(call), c("call", "frame")) &&
+        !identical(names(call), c("", "frame"))) {
+      abort("When a list, `call` must have \"call\" and \"frame\" names.")
+    }
+    .frame <- call[["frame"]] %||% .frame
+    call <- call[["call"]]
+  }
+
+  # `.frame` is used to soft-truncate the backtrace
   if (is_null(.frame)) {
-    if (is_environment(maybe_missing(call))) {
-      .frame <- call
-    } else {
+    if (rethrowing) {
       .frame <- caller
+    } else {
+      # Truncate backtrace up to `call` if it is a frame
+      if (is_environment(maybe_missing(call))) {
+        .frame <- call
+      } else {
+        .frame <- caller
+      }
     }
   } else {
     check_environment(.frame)
   }
 
-  info <- abort_context(.frame, parent)
+  info <- abort_context(.frame, rethrowing, maybe_missing(call))
 
   if (is_missing(call)) {
     if (is_null(info$from_handler)) {
@@ -246,6 +290,8 @@ abort <- function(message = NULL,
     } else {
       call <- info$setup_caller
     }
+  } else if (rethrowing && identical(call, info$handler_frame)) {
+    call <- info$setup_caller
   }
 
   message <- validate_signal_args(message, class, call, .subclass, "abort")
@@ -263,6 +309,9 @@ abort <- function(message = NULL,
   extra_fields <- message_info$extra_fields
   use_cli_format <- message_info$use_cli_format
 
+  if (rethrowing) {
+    trace <- trace %||% parent[["trace"]]
+  }
   if (is_null(trace) && is_null(peek_option("rlang:::disable_trace_capture"))) {
     with_options(
       # Prevents infloops when rlang throws during trace capture
@@ -285,55 +334,71 @@ abort <- function(message = NULL,
   signal_abort(cnd, .file)
 }
 
-abort_context <- function(frame, parent, call = caller_env()) {
+abort_context <- function(frame,
+                          rethrowing,
+                          abort_call,
+                          call = caller_env()) {
   calls <- sys.calls()
   frames <- sys.frames()
   parents <- sys.parents()
 
   frame_loc <- detect_index(frames, identical, frame)
+  bottom_loc <- frame_loc
   setup_loc <- 0L
+
   setup_caller <- NULL
   from_handler <- NULL
+  handler_frame <- NULL
 
-  if (frame_loc > 1L) {
-    prev_frame <- frames[[frame_loc - 1L]]
-    if (env_has(prev_frame, ".__handler_frame__.")) {
-      from_handler <- "calling"
-      frame_loc <- frame_loc - 1L
+  # If rethrowing we need to find:
+  # - The caller of the condition setup frame. This replaces `call`
+  #   when it points to the handler frame.
+  # - The caller of the handler frame, used to soft-truncate the
+  #   backtrace. This way we hide the condition signalling and
+  #   handling context (which can be quite complex) in simplified
+  #   backtraces.
+  if (rethrowing) {
 
-      setup_frame <- env_get(prev_frame, ".__setup_frame__.", default = NULL)
-      if (!is_null(setup_frame)) {
-        setup_caller <- eval_bare(call2(parent.frame), setup_frame)
-      }
-    }
-  }
+    # This iteration through callers may be incorrect in case of
+    # intervening frames. Ideally, we'd iterate only over parent frames.
+    # This shouldn't be likely to cause issues though.
+    while (is_null(from_handler) && frame_loc > 1L) {
+      prev_frame <- frames[[frame_loc - 1L]]
+      if (env_has(prev_frame, ".__handler_frame__.")) {
+        from_handler <- "calling"
+        handler_frame <- frames[[frame_loc]]
+        frame_loc <- frame_loc - 1L
 
-  bottom_loc <- frame_loc
-
-  if ((frame_loc - 1) > 0) {
-    # We currently have no reliable way of detecting a rethrow from a
-    # condition handler. We attempt to detect it from the call stack
-    # in three cases: (a) when we see a `tryCatch()` stack, (b) when a
-    # `parent` condition is supplied, (c) when a `.__handler_frame__.`
-    # binding is present in the calling frame (as determined by `call`
-    # if an environment).
-    call1 <- calls[[frame_loc]]
-    call2 <- calls[[frame_loc - 1]]
-
-    if (is_exiting_handler_call(call1, call2)) {
-      from_handler <- "exiting"
-      setup_loc <- calls_try_catch_loc(calls, frame_loc)
-      bottom_loc <- parents[[setup_loc]]
-    } else {
-      if (is_string(from_handler, "calling") || !is_null(parent)) {
-        if (is_calling_handler_inlined_call(call1)) {
-          from_handler <- "calling"
-          bottom_loc <- calls_signal_loc(calls, frame_loc - 1L)
-        } else if (is_calling_handler_simple_error_call(call1, call2)) {
-          from_handler <- "calling"
-          bottom_loc <- calls_signal_loc(calls, frame_loc - 2L)
+        setup_frame <- env_get(prev_frame, ".__setup_frame__.", default = NULL)
+        if (!is_null(setup_frame)) {
+          setup_caller <- eval_bare(call2(parent.frame), setup_frame)
         }
-        setup_loc <- calls_setup_loc(calls, frames, frame_loc)
+      }
+
+      if ((frame_loc - 1) > 0) {
+        call1 <- calls[[frame_loc]]
+        call2 <- calls[[frame_loc - 1]]
+
+        if (is_exiting_handler_call(call1, call2)) {
+          from_handler <- "exiting"
+          handler_frame <- handler_frame %||% frames[[frame_loc]]
+          setup_loc <- calls_try_catch_loc(calls, frame_loc)
+          bottom_loc <- parents[[setup_loc]]
+        } else {
+          if (is_calling_handler_inlined_call(call1)) {
+            from_handler <- "calling"
+            handler_frame <- handler_frame %||% frames[[frame_loc]]
+            bottom_loc <- calls_signal_loc(calls, frame_loc - 1L)
+          } else if (is_calling_handler_simple_error_call(call1, call2)) {
+            from_handler <- "calling"
+            handler_frame <- handler_frame %||% frames[[frame_loc]]
+            bottom_loc <- calls_signal_loc(calls, frame_loc - 2L)
+          }
+          setup_loc <- calls_setup_loc(calls, frames, frame_loc)
+        }
+      }
+      if (is_null(from_handler)) {
+        frame_loc <- frame_loc - 1L
       }
     }
   }
@@ -342,6 +407,12 @@ abort_context <- function(frame, parent, call = caller_env()) {
     # Skip frames marked with the sentinel `.__signal_frame__.`
     bottom_loc <- skip_signal_frames(bottom_loc, frames)
     bottom_frame <- frames[[bottom_loc]]
+    if (!rethrowing && !is_missing(abort_call) && is_environment(abort_call)) {
+      abort_call_loc <- detect_index(frames, identical, abort_call)
+      if (abort_call_loc && abort_call_loc < bottom_loc) {
+        bottom_frame <- frames[[abort_call_loc]]
+      }
+    }
   } else {
     bottom_frame <- NULL
   }
@@ -352,6 +423,7 @@ abort_context <- function(frame, parent, call = caller_env()) {
 
   list(
     from_handler = from_handler,
+    handler_frame = handler_frame,
     bottom_frame = bottom_frame,
     setup_caller = setup_caller
   )
@@ -460,7 +532,7 @@ skip_signal_frames <- function(loc, frames) {
 }
 
 is_calling_handler_inlined_call <- function(call) {
-  is_call(call) && is_function(call[[1]]) && is_condition(call[[2]])
+  is_call(call) && length(call) >= 2 && is_function(call[[1]]) && is_condition(call[[2]])
 }
 is_calling_handler_simple_error_call <- function(call1, call2) {
   identical(call1, quote(h(simpleError(msg, call)))) && is_call(call2, ".handleSimpleError")
@@ -512,7 +584,7 @@ cnd_message_info <- function(message,
       fields$footer <- footer
     }
     if (internal) {
-      fields$footer <- footer_internal
+      fields$footer <- footer_internal(env)
     }
   } else {
     # Compatibility with older bullets formatting
@@ -530,7 +602,7 @@ cnd_message_info <- function(message,
       fields$footer <- footer
     }
     if (internal) {
-      message <- c(message, footer_internal)
+      message <- c(message, footer_internal(env))
     }
     message <- .rlang_cli_format_fallback(message)
   }
@@ -543,9 +615,20 @@ cnd_message_info <- function(message,
 }
 utils::globalVariables(".internal")
 
-footer_internal <- c(
-  "i" = "This is an internal error, please report it to the package authors."
-)
+footer_internal <- function(env) {
+  top <- topenv(env)
+  if (is_namespace(top)) {
+    pkg <- sprintf(" in the %s package", ns_env_name(top))
+  } else {
+    pkg <- ""
+  }
+  footer <- sprintf(
+    "This is an internal error%s, please report it to the package authors.",
+    pkg
+  )
+
+  c(i = footer)
+}
 
 stop_multiple_body <- function(body, call) {
   msg <- c(
@@ -665,7 +748,7 @@ check_use_cli_flag <- function(flag, error_call) {
   }
 
   if (flag[["inline"]]) {
-    if (!has_cli_format || !has_cli_inline) {
+    if (!has_cli_format) {
       msg <- c(
         "`.__rlang_use_cli__.[[\"inline\"]]` is set to `TRUE` but cli is not installed or is too old.",
         "i" = "The package author should add a recent version of `cli` to their `Imports`."
@@ -1030,7 +1113,7 @@ error_call_as_string <- function(call) {
   #   all environments and the `sys.call()` for that frame is `eval()`.
   #   It wouldn't be useful to display this as the context so calls to
   #   `eval()` and `evalq()` are replaced by `NULL`.
-  if (is_call(call, c("eval", "evalq"))) {
+  if (is_call(call, c("eval", "evalq", "eval_tidy"))) {
     return(NULL)
   }
 
