@@ -1,8 +1,8 @@
 test_that("format_error_bullets() formats bullets depending on names", {
-  expect_identical(format_error_bullets(c("foo", "bar")), "* foo\n* bar")
-  expect_identical(format_error_bullets(c(i = "foo", "*" = "baz", x = "bar", v = "bam")), "i foo\n* baz\nx bar\nv bam")
-  expect_error(format_error_bullets(c(i = "foo", u = "bar")))
-  expect_identical(format_error_bullets(chr()), chr())
+  expect_equal(format_error_bullets(c("foo", "bar")), "* foo\n* bar")
+  expect_equal(format_error_bullets(c(i = "foo", "*" = "baz", x = "bar", v = "bam")), "i foo\n* baz\nx bar\nv bam")
+  expect_equal(format_error_bullets(c(i = "foo", u = "bar")), "i foo\nbar")
+  expect_equal(format_error_bullets(chr()), chr())
 })
 
 test_that("default conditionMessage() method for rlang errors calls cnd_message()", {
@@ -179,13 +179,6 @@ test_that("cli is not used when message is escaped with `I()`", {
   )
 })
 
-test_that("cli syntax is escaped in 'try' mode", {
-  local_use_cli()
-
-  x <- "{foo {{}}"
-  expect_equal(rlang_format_message(x), x)
-})
-
 test_that(".rlang_cli_str_restore() deals with attributes", {
   msg <- structure("foo", attr = TRUE)
 
@@ -199,24 +192,6 @@ test_that(".rlang_cli_str_restore() deals with attributes", {
     .rlang_cli_str_restore("bar", msg_oo),
     "bar"
   )
-
-  local_use_cli(inline = TRUE, format = TRUE)
-  expect_equal(
-    attributes(rlang_format_message(msg)),
-    list(attr = TRUE)
-  )
-
-  local_use_cli(inline = FALSE, format = FALSE)
-  expect_equal(
-    attributes(rlang_format_message(msg)),
-    list(attr = TRUE)
-  )
-
-  local_use_cli(inline = FALSE, format = TRUE)
-  expect_equal(
-    attributes(rlang_format_message(msg)),
-    list(attr = TRUE)
-  )
 })
 
 skip_if_not_installed("cli", "2.5.0")
@@ -490,3 +465,50 @@ test_that("multiline operator calls are preserved", {
   expect_snapshot_output(err({ 1; 2 } + { 2; 3 }))
   expect_snapshot_output(err(x[{ 1; 2 }]))
 })
+
+test_that("eval_tidy() is not mentioned in calls", {
+  expect_null(format_error_call(quote(eval_tidy(expr))))
+})
+
+test_that("header, body, and footer don't partial-match", {
+  expect_equal(
+    cnd_header(error_cnd("foo", headers = 1)),
+    ""
+  )
+  expect_equal(
+    cnd_body(error_cnd("foo", bodyy = 1)),
+    chr()
+  )
+  expect_equal(
+    cnd_footer(error_cnd("foo", footers = 1)),
+    chr()
+  )
+})
+
+test_that("can disable srcrefs in call formatting", {
+  withr::local_envvar(c("TESTTHAT" = "false"))
+  local_options(rlang_call_format_srcrefs = FALSE)
+
+  with_srcref("{
+    f <- function() { g() }
+    g <- function() abort('foo')
+  }")
+
+  expect_snapshot(err(f()))
+})
+
+test_that("fallback method supports unknown bullets (#1364)", {
+  local_use_cli(format = FALSE)
+  expect_snapshot({
+    "With fallback"
+    (expect_error(abort(c("foo", i2 = "bar"))))
+    (expect_error(abort(c(i1 = "foo", i2 = "bar"))))
+  })
+
+  local_use_cli(format = TRUE)
+  expect_snapshot({
+    "With cli"
+    (expect_error(abort(c("foo", i2 = "bar"))))
+    (expect_error(abort(c(i1 = "foo", i2 = "bar"))))
+  })
+})
