@@ -489,75 +489,6 @@ test_that("can trim layers of backtraces", {
   expect_equal_trace(trace3, trace3_env)
 })
 
-test_that("can subset from bottom tree level", {
-  idx <- int(0, 1, 1, 1)
-  expect_identical(chain_indices(4L, idx), c(1L, 4L))
-  expect_identical(parents_indices(4L, idx), c(1L, 4L))
-  expect_identical(children_indices(4L, idx), int())
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() trace_back(e)
-  trace <- f()
-  out <- trace_subset_across(trace, 3)
-
-  f <- function() g()
-  expected <- f()
-
-  expect_equal_trace(out, expected)
-})
-
-test_that("can subset from top tree level", {
-  idx <- int(0, 1, 0, 3, 3, 3)
-  expect_identical(chain_indices(3L, idx), 3:6)
-  expect_identical(parents_indices(3L, idx), 3L)
-  expect_identical(children_indices(3L, idx), 4:6)
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() trace_back(e)
-  trace <- tryCatch(f())
-
-  out <- trace_subset_across(trace, -1, 1)
-  expected <- f()
-
-  expect_equal_trace(out, expected)
-})
-
-test_that("can subset in middle level", {
-  idx <- int(0, 1, 1, 1, 4, 4, 4)
-  expect_identical(chain_indices(3L, idx), c(1L, 3L))
-  expect_identical(parents_indices(3L, idx), c(1L, 3L))
-  expect_identical(children_indices(3L, idx), int())
-
-  e <- current_env()
-  f <- function() identity(identity(g()))
-  g <- function() identity(identity(h()))
-  h <- function() trace_back(e)
-  trace <- f()
-
-  out <- trace_subset_across(trace, 2, 2)
-  expect_equal(out$call, alist(f(), identity(g())))
-  expect_identical(out$parent, 0:1)
-
-
-  idx <- int(0, 1, 1, 1, 4, 4, 4)
-  expect_identical(chain_indices(4L, idx), c(1L, 4:7))
-  expect_identical(parents_indices(4L, idx), c(1L, 4L))
-  expect_identical(children_indices(4L, idx), 5:7)
-
-  out <- trace_subset_across(trace, 3, 2)
-  exp <- alist(
-    f(),
-    g(),
-    identity(identity(h())),
-    identity(h()),
-    h()
-  )
-  expect_equal(out$call, exp)
-  expect_identical(out$parent, c(0L, 1L, 2L, 2L, 2L))
-})
-
 test_that("fails when `bottom` is not on the stack", {
   expect_error(trace_back(bottom = env()), "Can't find `bottom`")
 })
